"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.updatePackageJson = void 0;
const path_1 = require("path");
const builder_1 = require("@build-script/builder");
const node_json_edit_1 = require("@idlebox/node-json-edit");
const unipm_1 = require("unipm");
const argParse_1 = require("../inc/argParse");
const paths_1 = require("../inc/paths");
async function updatePackageJson({ isESNext, hookMode, dualPackage, outDir }) {
    const configRel = paths_1.relativePosix(argParse_1.PROJECT_ROOT, argParse_1.CONFIG_FILE);
    const packageJson = await node_json_edit_1.loadJsonFile(path_1.resolve(argParse_1.PROJECT_ROOT, 'package.json'));
    const relOut = './' + path_1.posix.relative(argParse_1.PROJECT_ROOT, outDir + '/' + paths_1.INDEX_FILE_NAME);
    if (isESNext) {
        node_json_edit_1.insertKeyAlphabet(packageJson, 'type', 'module');
        if (dualPackage) {
            console.log('inserting dual package.');
            node_json_edit_1.insertKeyAlphabet(packageJson, 'main', relOut + '.cjs');
            node_json_edit_1.insertKeyAlphabet(packageJson, 'module', relOut + '.js');
            if (!packageJson.exports) {
                node_json_edit_1.insertKeyAlphabet(packageJson, 'exports', {});
            }
            if (!packageJson.exports['.']) {
                packageJson.exports['.'] = {};
            }
            packageJson.exports['.']['require'] = relOut + '.cjs';
            packageJson.exports['.']['import'] = relOut + '.js';
        }
        else {
            node_json_edit_1.insertKeyAlphabet(packageJson, 'main', relOut + '.js');
        }
    }
    else {
        node_json_edit_1.insertKeyAlphabet(packageJson, 'type', 'commonjs');
        node_json_edit_1.insertKeyAlphabet(packageJson, 'main', relOut + '.js');
    }
    node_json_edit_1.insertKeyAlphabet(packageJson, 'typings', 'docs/package-public.d.ts');
    if (hookMode) {
        const myFile = '@build-script/export-all-in-one/build-script-register';
        const previous = (await builder_1.getPlugin(myFile)) || [];
        await builder_1.registerPlugin(myFile, [dualPackage ? '' : argParse_1.NO_DUAL_FLAG, ...previous, configRel].filter(uniqNotEmpty));
    }
    else {
        if (!packageJson.scripts) {
            node_json_edit_1.insertKeyAlphabet(packageJson, 'scripts', {});
        }
        if (!packageJson.scripts['build:export-all-in-one']) {
            node_json_edit_1.insertKeyAlphabet(packageJson.scripts, 'export-all-in-one', 'export-all-in-one ' + (dualPackage ? argParse_1.NO_DUAL_FLAG + ' ' : '') + configRel);
        }
    }
    if (!packageJson.devDependencies || !packageJson.devDependencies['@build-script/export-all-in-one']) {
        const oldFormat = node_json_edit_1.getFormatInfo(await node_json_edit_1.loadJsonFile(path_1.resolve(argParse_1.PROJECT_ROOT, 'package.json')));
        const pm = await unipm_1.getPackageManager({ cwd: argParse_1.PROJECT_ROOT });
        await pm.install('--dev', '@build-script/export-all-in-one');
        const packageJson = require(path_1.resolve(argParse_1.PROJECT_ROOT, 'package.json'));
        node_json_edit_1.reformatJson(packageJson, oldFormat);
    }
    await node_json_edit_1.writeJsonFileBack(packageJson);
}
exports.updatePackageJson = updatePackageJson;
function uniqNotEmpty(item, index, self) {
    return item && index === self.lastIndexOf(item);
}
//# sourceMappingURL=updatePackageJson.js.map